unit DecEdit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TNumericalType = (DecimalUnSigned, DecimalSigned, DoubleUnSigned, DoubleSigned);
  TDecEdit = class(TEdit)
  private
    FAlignment: TAlignment;
    FNumericType: TNumericalType;
    procedure SetAlignment(Value: TAlignment);
    procedure SetNumericType(Value: TNumericalType);
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure KeyPress(var Key: Char); Override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Alignment: TAlignment  read FAlignment  write SetAlignment  default taLeftJustify;
    property NumericType: TNumericalType  read FNumericType  write SetNumericType  default DecimalUnSigned;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TDecEdit]);
end;

procedure TDecEdit.KeyPress(var Key: Char);
var
  i: Integer;
  f: Integer;
begin
  if FNumericType = DecimalUnSigned then
  begin
    {#8 BS}
    if not SysUtils.CharInSet(key, ['0'..'9', #8]) then
    begin
      Key := #0 ;
    end
    else
    begin
      inherited;
    end;
  end;

  if FNumericType = DecimalSigned then
  begin
    {#8 BS}
    if not SysUtils.CharInSet(key, ['0'..'9', #8, '-']) then
    begin
      Key := #0 ;
    end
    else
    begin
      {Only top pos}
      if key = '-' then
      begin
        if Selstart = 0 then
        begin
          inherited;
        end
        else
        begin
          Key := #0 ;
        end;
      end
      else
      begin
        inherited;
      end;
    end;
  end;

  if FNumericType = DoubleUnSigned then
  begin
    {#8 BS}
    if not SysUtils.CharInSet(key, ['0'..'9', #8, '.']) then
    begin
      Key := #0 ;
    end
    else
    begin
      {Only one}
      if key = '.' then
      begin
        f := -1;
        for i := 0 to Length(Text)-1 do
        begin
          if Copy(Text, i+1, 1) = '.' then
          begin
            f := i;
            Break;
          end;
        end;
        if f = -1 then
        begin
          inherited;
        end
        else
        begin
          Key := #0 ;
        end;
      end
      else
      begin
        inherited;
      end;
    end;
  end;

  if FNumericType = DoubleSigned then
  begin
    {#8 BS}
    if not SysUtils.CharInSet(key, ['0'..'9', #8, '.', '-']) then
    begin
      Key := #0 ;
    end
    else
    begin
      {Only top pos}
      if key = '-' then
      begin
        if Selstart = 0 then
        begin
          inherited;
        end
        else
        begin
          Key := #0 ;
        end;
      end
      {Only one}
      else if key = '.' then
      begin
        f := -1;
        for i := 0 to Length(Text)-1 do
        begin
          if Copy(Text, i+1, 1) = '.' then
          begin
            f := i;
            Break;
          end;
        end;
        if f = -1 then
        begin
          inherited;
        end
        else
        begin
          Key := #0 ;
        end;
      end
      else
      begin
        inherited;
      end;
    end;
  end;
end;

constructor TDecEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FNumericType := DecimalUnSigned;
end;

procedure TDecEdit.CreateParams(var Params: TCreateParams);
const
  Alignments: array [TAlignment] of Longint = (ES_LEFT, ES_RIGHT, ES_CENTER);
begin
  inherited CreateParams(Params);
  with Params do
  begin
    {D2}
    {$IFDEF VER90}
      Style := Style or Alignments[FAlignment] or ES_MULTILINE;
    {$ENDIF}

    {D4}
    {$IFDEF VER120}
      DWORD(Style) := DWORD(Style) or DWORD(Alignments[FAlignment]) or DWORD(ES_MULTILINE);
    {$ENDIF}

    {D6}
    {$IFDEF VER140}
      DWORD(Style) := DWORD(Style) or DWORD(Alignments[FAlignment]) or DWORD(ES_MULTILINE);
    {$ENDIF}

    {D7}
    {$IFDEF VER160}
      DWORD(Style) := DWORD(Style) or DWORD(Alignments[FAlignment]) or DWORD(ES_MULTILINE);
      Style := Style or Alignments[FAlignment] or ES_MULTILINE;
    {$ENDIF}
  end;
end;

procedure TDecEdit.SetNumericType(Value: TNumericalType);
begin
  if FNumericType <> Value then
  begin
    FNumericType := Value;
  end;
end;

procedure TDecEdit.SetAlignment(Value: TAlignment);
begin
  if FAlignment <> Value then
  begin
    FAlignment := Value;
    RecreateWnd;
  end;
end;

end.
