﻿// function SearchText
// Start  : 1997/07/05
// UpDate : 2001/07/25
// Copyright  (C)  本田勝彦  <vyr01647@niftyserve.or.jp>
// Modified DEKO          2010/02/15
// Modified Takeshi Oishi 2019/01/19

unit HtSearch;

interface

uses
  SysUtils, Windows, Classes, StdCtrls, Dialogs,
  Vcl.Forms, System.Variants,
  //DEKO さんが公開している
  MecsUtils,
  //XE2 の標準の機能を使う
  System.RegularExpressions,
  //
  MemoExDark;

const
  //文字数
  SEARCH_FOEWARD_LENGTH_11 = 10000;

type
  TSearchOption = (sfrDown, sfrMatchCase, sfrWholeWord,
                   sfrNoMatchZenkaku,
                   sfrReplace, sfrReplaceAll, sfrReplaceConfirm,
                   sfrIncludeCRLF, sfrIncludeSpace,
                   sfrRegExp);
  TSearchOptions = set of TSearchOption;

  TSearchInfo =
    record
      Start: Integer;
      Length: Integer;
    end;

function SearchText(TextLine: PChar;
                    var Info: TSearchInfo;
                    const SearchString: String;
                    Options: TSearchOptions): Boolean;

function SearchText1(TextLine: PWideChar;
                    var Info: TSearchInfo;
                    const SearchString: String;
                    Options: TSearchOptions;
                    MemoEx: TMemoExDark): Boolean;

function SearchText2(TextLine: PWideChar;
                    var Info: TSearchInfo;
                    const SearchString: String;
                    Options: TSearchOptions;
                    MemoEx: TMemoExDark): Boolean;

function SearchText5(TextLine: PChar;
                    var Info: TSearchInfo;
                    const SearchString: String;
                    Options: TSearchOptions;
                    var StrList: TStringList
                    ): Boolean;

type
  TStringsSearchInfo =
    record
      Line: Integer;
      Column: Integer;
      Length: Integer;
    end;

function SearchStrings(Strings: TStrings;
                       var StringInfo: TStringsSearchInfo;
                       const SearchString: String;
                       Options: TSearchOptions;
                       MemoEx: TMemoExDark): Boolean;


implementation

uses
  Main;

function SearchBuf(Buf: PChar;
                   var Info: TSearchInfo;
                   SearchString: String;
                   Options: TSearchOptions): PChar;
var
  I: Integer;

  sDmy1: String;
  sSkip1: String;
  sSearch: String;

  iBufLen1: Integer;
  iSkipFlag1: Integer;

  RegExp: TRegEx;
  Match: TMatch;
  RegOptions: TRegExOptions;
  SearchInfo: TSearchInfo;

  //関数内関数
  //AltSearchString Begin
  function AltSearchString(S: String): String;
  var
    I: Integer;
  begin
    //結果
    result := '';
    //結合
    for I := Length(S) downto 1 do
    begin
      case S[I] of
        #$21 .. #$2F, #$3A .. #$40, #$5B .. #$60, #$7B .. #$7E:
          result := '\' + S[I] + result;
        #$07:
          result := '\a' + result;
        // #$08:
        // result := '\b' + result;
        #$09:
          result := '\t' + result;
        #$0A:
          result := '\n' + result;
        #$0B:
          result := '\v' + result;
        #$0C:
          result := '\f' + result;
        #$0D:
          result := '\r' + result;
        #$1b:
          result := '\e' + result;
      else
        result := S[I] + result;
      end;
    end;
  end;
  //AltSearchString End

begin
  //結果
  result := nil;
  //取得
  iBufLen1 := SysUtils.StrLen(Buf);
  //判定
  if (Info.Start < 0) or (Info.Start > iBufLen1) or (Info.Length < 0) then
  begin
    Exit;
  end;

  try
    //正規表現形式を使う
    if (sfrRegExp in Options) then
    begin
      //正規表現検索
      //・改行を無視     (sfrIncludeCRLF)
      //・空白を無視     (sfrIncludeSpace)
      //・ワード単位検索 (sfrWholeWord)
      //これらは、正規表現の構文で指定出来るため無視される(処理がカブる)。
      //検索文字列はそのまま渡される。
      //検索方向に sfrDown が指定されない場合は逆順で検索(上方検索)されるが、
      //本来、正規表現検索には逆順(上方)検索は存在しない(ハズ)。
      //便宜上、逆順(上方)検索が使える事にしているが、
      //よく考えてみると正規表現パターンと一致した文字列の辻褄が合わない事が
      //解ると思う。

      //正規表現で実現するので、オプションから削除する
      Options := Options - [sfrIncludeCRLF, sfrIncludeSpace, sfrWholeWord];
      //検索文字列
      sSearch := SearchString;
    end

    //正規表現に変換して使う
    else
    begin
      //通常の検索
      //検索文字列は正規表現パターンに変換されて渡される。
      //通常の検索と正規表現検索での検索結果が異なるのを防ぐため、
      //どちらも SkRegExp に渡している。
      //オプションを設定
      iSkipFlag1 := Integer(sfrIncludeCRLF  in Options) or
                 (Integer(sfrIncludeSpace in Options) shl 1);
      case iSkipFlag1 of
        1: //改行無視
          sSkip1 := '[\r\n]*';
        2: //空白無視
          sSkip1 := '\x20*';
        3: //改行/空白無視
          sSkip1 := '([\r\n]*|\x20*)';
      else
        sSkip1 := '';
      end;

      //上記のオプション分を加える
      if iSkipFlag1 > 0 then
      begin
        sSearch := '';
        sDmy1 := SearchString;
        for I := 1 to MecsLength(sDmy1) do
        begin
          sSearch := sSearch + AltSearchString(MecsCopy(sDmy1, I, 1));
          if I < MecsLength(sDmy1) then
          begin
            sSearch := sSearch + sSkip1;
          end;
        end;
      end
      //オプションが無い
      else
      begin
        sSearch := AltSearchString(SearchString);
      end;

      //単語単位
      if (sfrWholeWord in Options) then
      begin
        sSearch := '\b' + sSearch + '\b';
      end;
    end;


    //設定
    if (sfrMatchCase in Options) then
    begin
      //
    end
    else
    begin
      RegOptions := RegOptions + [roIgnoreCase];
    end;

    //設定
    if (sfrIncludeCRLF in Options) then
    begin
      RegOptions := RegOptions + [roMultiline];
    end
    else
    begin
      //
    end;

    //確保
    //パターン文字列とオプションを指定
    RegExp := TRegEx.Create(sSearch, RegOptions);

    //結果
    result := nil;

    //Match.index は 1 ベース

    //検索方向は無条件に文末方向のみ
      //範囲を判定
      if Info.Start >= iBufLen1 then
      begin
        Exit;
      end;

      /////////////////////////////////
      //ここだけ変更している
      //設定
      SearchInfo.Start := Info.Start + 0;
      SearchInfo.Length := 0;
      //SearchInfo.Start := Info.Start + 2;
      //SearchInfo.Length := 0;
      /////////////////////////////////

      //取得
      Match := RegExp.Match(StrPas(Buf), SearchInfo.Start);
      //発見
      if Match.Success = TRUE then
      begin
        //結果を返す
        Info.Start  := Match.Index - 1;
        Info.Length := Match.Length;
        result := PChar(@Buf[Info.Start]);
      end;


  finally
    //
  end;
end;

function SearchBuf1(Buf: PChar;
                   var Info: TSearchInfo;
                   SearchString: String;
                   Options: TSearchOptions;
                   MemoEx: TMemoExDark): PChar;
var
  I: Integer;

  sDmy1: String;
  sSkip1: String;
  sSearch: String;
  sFindResult: String;

  iBufLen1: Integer;

  iSkipFlag1: Integer;
  iRangeStart1: Integer;
  iRangeEnd1: Integer;
  iSearchStart1: Integer;
  iSearchEnd1: Integer;

  RegExp: TRegEx;
  Match: TMatch;
  Matches: TMatchCollection;
  RegOptions: TRegExOptions;
  SearchInfo: TSearchInfo;
  SearchInfo1: TSearchInfo;

  iRow1: Integer;
  iFind1: Integer;

  //関数内関数
  //AltSearchString Begin
  function AltSearchString(S: String): String;
  var
    I: Integer;
  begin
    //結果
    result := '';
    //結合
    for I := Length(S) downto 1 do
    begin
      case S[I] of
        #$21 .. #$2F, #$3A .. #$40, #$5B .. #$60, #$7B .. #$7E:
          result := '\' + S[I] + result;
        #$07:
          result := '\a' + result;
        // #$08:
        // result := '\b' + result;
        #$09:
          result := '\t' + result;
        #$0A:
          result := '\n' + result;
        #$0B:
          result := '\v' + result;
        #$0C:
          result := '\f' + result;
        #$0D:
          result := '\r' + result;
        #$1b:
          result := '\e' + result;
      else
        result := S[I] + result;
      end;
    end;
  end;
  //AltSearchString End

begin
  //結果
  result := nil;
  //取得
  iBufLen1 := SysUtils.StrLen(Buf);
  //判定
  if (Info.Start < 0) or (Info.Start > iBufLen1) or (Info.Length < 0) then
  begin
    Exit;
  end;

  try
    //正規表現形式を使う
    if (sfrRegExp in Options) then
    begin
      //正規表現検索
      //・改行を無視     (sfrIncludeCRLF)
      //・空白を無視     (sfrIncludeSpace)
      //・ワード単位検索 (sfrWholeWord)
      //これらは、正規表現の構文で指定出来るため無視される(処理がカブる)。
      //検索文字列はそのまま渡される。
      //検索方向に sfrDown が指定されない場合は逆順で検索(上方検索)されるが、
      //本来、正規表現検索には逆順(上方)検索は存在しない(ハズ)。
      //便宜上、逆順(上方)検索が使える事にしているが、
      //よく考えてみると正規表現パターンと一致した文字列の辻褄が合わない事が
      //解ると思う。

      //正規表現で実現するので、オプションから削除する
      Options := Options - [sfrIncludeCRLF, sfrIncludeSpace, sfrWholeWord];
      //検索文字列
      sSearch := SearchString;
    end

    //正規表現に変換して使う
    else
    begin
      //通常の検索
      //検索文字列は正規表現パターンに変換されて渡される。
      //通常の検索と正規表現検索での検索結果が異なるのを防ぐため、
      //どちらも SkRegExp に渡している。
      //オプションを設定
      iSkipFlag1 := Integer(sfrIncludeCRLF  in Options) or
                 (Integer(sfrIncludeSpace in Options) shl 1);
      case iSkipFlag1 of
        1: //改行無視
          sSkip1 := '[\r\n]*';
        2: //空白無視
          sSkip1 := '\x20*';
        3: //改行/空白無視
          sSkip1 := '([\r\n]*|\x20*)';
      else
        sSkip1 := '';
      end;

      //上記のオプション分を加える
      if iSkipFlag1 > 0 then
      begin
        sSearch := '';
        sDmy1 := SearchString;
        for I := 1 to MecsLength(sDmy1) do
        begin
          sSearch := sSearch + AltSearchString(MecsCopy(sDmy1, I, 1));
          if I < MecsLength(sDmy1) then
          begin
            sSearch := sSearch + sSkip1;
          end;
        end;
      end
      //オプションが無い
      else
      begin
        sSearch := AltSearchString(SearchString);
      end;

      //単語単位
      if (sfrWholeWord in Options) then
      begin
        sSearch := '\b' + sSearch + '\b';
      end;
    end;


    //設定
    if (sfrMatchCase in Options) then
    begin
      //
    end
    else
    begin
      RegOptions := RegOptions + [roIgnoreCase];
    end;

    //設定
    if (sfrIncludeCRLF in Options) then
    begin
      RegOptions := RegOptions + [roMultiline];
    end
    else
    begin
      //
    end;

    //確保
    //パターン文字列とオプションを指定
    RegExp := TRegEx.Create(sSearch, RegOptions);

    //結果
    result := nil;

    //Match.index は 1 ベース


    //文頭方向
    begin
      //最初に有無を判定する

      //初期化
      SearchInfo1.Start  := 0;
      SearchInfo1.Length := 0;
      //取得
      Match := RegExp.Match(StrPas(Buf), SearchInfo1.Start);
      //発見
      if Match.Success = TRUE then
      begin
        //結果が現在位置より大きい
        if (Match.Index - 1) >= Info.Start then
        begin
          Exit;
        end
      end
      //未発見
      else
      begin
        Exit;
      end;

      //一定量だけ戻した位置から検索する

      //記憶
      iSearchStart1 := 0;
      iSearchEnd1   := Info.Start - 1;
      //終了位置
      iRangeEnd1 := iSearchEnd1;
      //開始位置
      iRangeStart1 := iSearchEnd1 - SEARCH_FOEWARD_LENGTH_11 - Length(sSearch);
      //2019-08-23
      if iRangeStart1 < 0 then
      begin
        iRangeStart1 := 0;
      end;

      if iRangeStart1 < iSearchStart1 then
      begin
        iRangeStart1 := iSearchStart1;
      end;
      //検索位置
      SearchInfo.Start := iRangeStart1;

      //検索
      while TRUE do
      begin
        //取得
        Match := RegExp.Match(StrPas(Buf), SearchInfo.Start);
        //発見
        if Match.Success = TRUE then
        begin
          //結果が現在位置より小さい
          if (Match.Index - 1) <= iRangeEnd1 then
          begin
            //発見
            Break;
          end
          //結果が現在位置より大きい
          else
          begin
            //終了位置
            iRangeEnd1 := iRangeStart1;
            //開始位置
            iRangeStart1 := iRangeStart1 - SEARCH_FOEWARD_LENGTH_11 - Length(sSearch);
            //2019-08-23
            if iRangeStart1 < 0 then
            begin
              iRangeStart1 := 0;
            end;

            if iRangeStart1 < iSearchStart1 then
            begin
              iRangeStart1 := iSearchStart1;
            end;
            //検索位置
            SearchInfo.Start := iRangeStart1;
          end;
        end
        //未発見
        else
        begin
            //終了位置
            iRangeEnd1 := iRangeStart1;
            //開始位置
            iRangeStart1 := iRangeStart1 - SEARCH_FOEWARD_LENGTH_11 - Length(sSearch);
            //2019-08-23
            if iRangeStart1 < 0 then
            begin
              iRangeStart1 := 0;
            end;

            if iRangeStart1 < iSearchStart1 then
            begin
              iRangeStart1 := iSearchStart1;
            end;
            //検索位置
            SearchInfo.Start := iRangeStart1;
        end;
      end;

      //範囲の文字列を設定
      //ここでは検索文字列分を加える
      sFindResult := Copy(StrPas(Buf), iRangeStart1+1, iRangeEnd1 - iRangeStart1 + Length(sSearch));

      //一覧を取得
      Matches := RegExp.Matches(sFindResult);
      //検索
//      iFind1 := -1;
//    2020-11-18 先頭文字でエラーになる対応
      iFind1 := 0;
      for iRow1 := 0 to Matches.Count-1 do
      begin
        //発見位置が現在位置より小さい間は繰り返す
        if (iRangeStart1 + Matches.Item[iRow1].Index) <= MemoEx.SelStart then
        begin
          iFind1 := iRow1;
        end
        else
        begin
          Break;
        end;
      end;

      //結果を返す
      Info.Start  := iRangeStart1 + Matches.Item[iFind1].Index - 1;
      Info.Length := Matches.Item[iFind1].Length;
      result := PChar(@Buf[Info.Start]);

    end;
    //文頭方向

  finally
    //
  end;
end;



function SearchBuf2(Buf: PChar;
                   var Info: TSearchInfo;
                   SearchString: String;
                   Options: TSearchOptions;
                   MemoEx: TMemoExDark): PChar;
var
  I: Integer;

  sDmy1: String;
  sSkip1: String;
  sSearch: String;

  iBufLen1: Integer;

  iSkipFlag1: Integer;

  RegExp: TRegEx;
  Match: TMatch;
  RegOptions: TRegExOptions;
  SearchInfo: TSearchInfo;

  //関数内関数
  //AltSearchString Begin
  function AltSearchString(S: String): String;
  var
    I: Integer;
  begin
    //結果
    result := '';
    //結合
    for I := Length(S) downto 1 do
    begin
      case S[I] of
        #$21 .. #$2F, #$3A .. #$40, #$5B .. #$60, #$7B .. #$7E:
          result := '\' + S[I] + result;
        #$07:
          result := '\a' + result;
        // #$08:
        // result := '\b' + result;
        #$09:
          result := '\t' + result;
        #$0A:
          result := '\n' + result;
        #$0B:
          result := '\v' + result;
        #$0C:
          result := '\f' + result;
        #$0D:
          result := '\r' + result;
        #$1b:
          result := '\e' + result;
      else
        result := S[I] + result;
      end;
    end;
  end;
  //AltSearchString End

begin
  //結果
  result := nil;
  //取得
  iBufLen1 := SysUtils.StrLen(Buf);
  //判定
  if (Info.Start < 0) or (Info.Start > iBufLen1) or (Info.Length < 0) then
  begin
    Exit;
  end;

  try
    //正規表現形式を使う
    if (sfrRegExp in Options) then
    begin
      //正規表現検索
      //・改行を無視     (sfrIncludeCRLF)
      //・空白を無視     (sfrIncludeSpace)
      //・ワード単位検索 (sfrWholeWord)
      //これらは、正規表現の構文で指定出来るため無視される(処理がカブる)。
      //検索文字列はそのまま渡される。
      //検索方向に sfrDown が指定されない場合は逆順で検索(上方検索)されるが、
      //本来、正規表現検索には逆順(上方)検索は存在しない(ハズ)。
      //便宜上、逆順(上方)検索が使える事にしているが、
      //よく考えてみると正規表現パターンと一致した文字列の辻褄が合わない事が
      //解ると思う。

      //正規表現で実現するので、オプションから削除する
      Options := Options - [sfrIncludeCRLF, sfrIncludeSpace, sfrWholeWord];
      //検索文字列
      sSearch := SearchString;
    end

    //正規表現に変換して使う
    else
    begin
      //通常の検索
      //検索文字列は正規表現パターンに変換されて渡される。
      //通常の検索と正規表現検索での検索結果が異なるのを防ぐため、
      //どちらも SkRegExp に渡している。
      //オプションを設定
      iSkipFlag1 := Integer(sfrIncludeCRLF  in Options) or
                 (Integer(sfrIncludeSpace in Options) shl 1);
      case iSkipFlag1 of
        1: //改行無視
          sSkip1 := '[\r\n]*';
        2: //空白無視
          sSkip1 := '\x20*';
        3: //改行/空白無視
          sSkip1 := '([\r\n]*|\x20*)';
      else
        sSkip1 := '';
      end;

      //上記のオプション分を加える
      if iSkipFlag1 > 0 then
      begin
        sSearch := '';
        sDmy1 := SearchString;
        for I := 1 to MecsLength(sDmy1) do
        begin
          sSearch := sSearch + AltSearchString(MecsCopy(sDmy1, I, 1));
          if I < MecsLength(sDmy1) then
          begin
            sSearch := sSearch + sSkip1;
          end;
        end;
      end
      //オプションが無い
      else
      begin
        sSearch := AltSearchString(SearchString);
      end;

      //単語単位
      if (sfrWholeWord in Options) then
      begin
        sSearch := '\b' + sSearch + '\b';
      end;
    end;


    //設定
    if (sfrMatchCase in Options) then
    begin
      //
    end
    else
    begin
      RegOptions := RegOptions + [roIgnoreCase];
    end;

    //設定
    if (sfrIncludeCRLF in Options) then
    begin
      RegOptions := RegOptions + [roMultiline];
    end
    else
    begin
      //
    end;

    //確保
    //パターン文字列とオプションを指定
    RegExp := TRegEx.Create(sSearch, RegOptions);

    //結果
    result := nil;

    //Match.index は 1 ベース


    //文末方向
    if (sfrDown in Options) then
    begin
      //範囲を判定
      if Info.Start >= Length(MemoEx.Text) then
      begin
        Exit;
      end;

      /////////////////////////////////
      //ここだけ変更している
      //設定
      SearchInfo.Start := Info.Start + 0;
      SearchInfo.Length := 0;
      //SearchInfo.Start := Info.Start + 2;
      //SearchInfo.Length := 0;
      /////////////////////////////////

      //取得
      Match := RegExp.Match(StrPas(Buf), SearchInfo.Start);
      //発見
      if Match.Success = TRUE then
      begin
        //結果を返す
        Info.Start  := Match.Index - 1;
        Info.Length := Match.Length;
        result := PChar(@Buf[Info.Start]);
      end;

    end;
    //文末方向

  finally
    //
  end;
end;

function SearchBuf5(Buf: PChar;
                   var Info: TSearchInfo;
                   SearchString: String;
                   Options: TSearchOptions;
                   var StrList: TStringList
                   ): PChar;
var
  I: Integer;

  sDmy1: String;
  sSkip1: String;
  sSearch: String;

  iBufLen1: Integer;
  iSkipFlag1: Integer;
  iRow1: Integer;

  RegExp: TRegEx;
  Matches: TMatchCollection;
  RegOptions: TRegExOptions;

  //関数内関数
  //AltSearchString Begin
  function AltSearchString(S: String): String;
  var
    I: Integer;
  begin
    //結果
    result := '';
    //結合
    for I := Length(S) downto 1 do
    begin
      case S[I] of
        #$21 .. #$2F, #$3A .. #$40, #$5B .. #$60, #$7B .. #$7E:
          result := '\' + S[I] + result;
        #$07:
          result := '\a' + result;
        // #$08:
        // result := '\b' + result;
        #$09:
          result := '\t' + result;
        #$0A:
          result := '\n' + result;
        #$0B:
          result := '\v' + result;
        #$0C:
          result := '\f' + result;
        #$0D:
          result := '\r' + result;
        #$1b:
          result := '\e' + result;
      else
        result := S[I] + result;
      end;
    end;
  end;
  //AltSearchString End

begin
  //結果
  result := nil;
  //取得
  iBufLen1 := SysUtils.StrLen(Buf);
  //判定
  if (Info.Start < 0) or (Info.Start > iBufLen1) or (Info.Length < 0) then
  begin
    Exit;
  end;

  try
    //正規表現形式を使う
    if (sfrRegExp in Options) then
    begin
      //正規表現検索
      //・改行を無視     (sfrIncludeCRLF)
      //・空白を無視     (sfrIncludeSpace)
      //・ワード単位検索 (sfrWholeWord)
      //これらは、正規表現の構文で指定出来るため無視される(処理がカブる)。
      //検索文字列はそのまま渡される。
      //検索方向に sfrDown が指定されない場合は逆順で検索(上方検索)されるが、
      //本来、正規表現検索には逆順(上方)検索は存在しない(ハズ)。
      //便宜上、逆順(上方)検索が使える事にしているが、
      //よく考えてみると正規表現パターンと一致した文字列の辻褄が合わない事が
      //解ると思う。

      //正規表現で実現するので、オプションから削除する
      Options := Options - [sfrIncludeCRLF, sfrIncludeSpace, sfrWholeWord];
      //検索文字列
      sSearch := SearchString;
    end

    //正規表現に変換して使う
    else
    begin
      //通常の検索
      //検索文字列は正規表現パターンに変換されて渡される。
      //通常の検索と正規表現検索での検索結果が異なるのを防ぐため、
      //どちらも SkRegExp に渡している。
      //オプションを設定
      iSkipFlag1 := Integer(sfrIncludeCRLF  in Options) or
                 (Integer(sfrIncludeSpace in Options) shl 1);
      case iSkipFlag1 of
        1: //改行無視
          sSkip1 := '[\r\n]*';
        2: //空白無視
          sSkip1 := '\x20*';
        3: //改行/空白無視
          sSkip1 := '([\r\n]*|\x20*)';
      else
        sSkip1 := '';
      end;

      //上記のオプション分を加える
      if iSkipFlag1 > 0 then
      begin
        sSearch := '';
        sDmy1 := SearchString;
        for I := 1 to MecsLength(sDmy1) do
        begin
          sSearch := sSearch + AltSearchString(MecsCopy(sDmy1, I, 1));
          if I < MecsLength(sDmy1) then
          begin
            sSearch := sSearch + sSkip1;
          end;
        end;
      end
      //オプションが無い
      else
      begin
        sSearch := AltSearchString(SearchString);
      end;

      //単語単位
      if (sfrWholeWord in Options) then
      begin
        sSearch := '\b' + sSearch + '\b';
      end;
    end;


    //設定
    if (sfrMatchCase in Options) then
    begin
      //
    end
    else
    begin
      RegOptions := RegOptions + [roIgnoreCase];
    end;

    //設定
    if (sfrIncludeCRLF in Options) then
    begin
      RegOptions := RegOptions + [roMultiline];
    end
    else
    begin
      //
    end;

    //確保
    //パターン文字列とオプションを指定
    RegExp := TRegEx.Create(sSearch, RegOptions);

    //結果
    result := nil;

    //Match.index は 1 ベース

    //検索方向は無条件に文末方向のみ
    //範囲を判定
    if Info.Start >= iBufLen1 then
    begin
      Exit;
    end;

    //一覧を取得
    Matches := RegExp.Matches(StrPas(Buf));
    //検索
    for iRow1 := 0 to Matches.Count-1 do
    begin
      if iRow1 = 0 then
      begin
        //結果を返す
        Info.Start  := Matches.Item[iRow1].Index-1;
        Info.Length := Matches.Item[iRow1].Length;
        result := PChar(@Buf[Info.Start]);
      end;
      StrList.Add(IntToStr(Matches.Item[iRow1].Index-1));
    end;
  finally
    //
  end;
end;



function SearchText(TextLine: PChar;
                    var Info: TSearchInfo;
                    const SearchString: String;
                    Options: TSearchOptions): Boolean;
var
  P: PChar;
begin
  //結果
  result := FALSE;
  //判定
  if (Length(SearchString) = 0) or (SysUtils.StrLen(TextLine) = 0) then
  begin
    Exit;
  end;

  //検索
  P := SearchBuf(TextLine, Info, SearchString, Options);
  //該当あり
  if P <> nil then
  begin
    //Info.Length は SearchBuf 内でセットされる
    Info.Start := P - TextLine;
    result := TRUE;
  end;
end;


//前方向の検索に使う
function SearchText1(TextLine: PWideChar;
                    var Info: TSearchInfo;
                    const SearchString: String;
                    Options: TSearchOptions;
                    MemoEx: TMemoExDark): Boolean;
var
  P: PChar;
begin
  //結果
  result := FALSE;
  //判定
  if (Length(SearchString) = 0) or (SysUtils.StrLen(TextLine) = 0) then
  begin
    Exit;
  end;

  //検索
  P := SearchBuf1(TextLine, Info, SearchString, Options, MemoEx);
  //該当あり
  if P <> nil then
  begin
    //Info.Length は SearchBuf 内でセットされる
    Info.Start := P - TextLine;
    result := TRUE;
  end;
end;

//次方向の検索に使う
function SearchText2(TextLine: PWideChar;
                    var Info: TSearchInfo;
                    const SearchString: String;
                    Options: TSearchOptions;
                    MemoEx: TMemoExDark): Boolean;
var
  P: PChar;
begin
  //結果
  result := FALSE;
  //判定
  if (Length(SearchString) = 0) or (SysUtils.StrLen(TextLine) = 0) then
  begin
    Exit;
  end;

  //検索
  P := SearchBuf2(TextLine, Info, SearchString, Options, MemoEx);
  //該当あり
  if P <> nil then
  begin
    //Info.Length は SearchBuf 内でセットされる
    Info.Start := P - TextLine;
    result := TRUE;
  end;
end;



function SearchText5(TextLine: PChar;
                    var Info: TSearchInfo;
                    const SearchString: String;
                    Options: TSearchOptions;
                    var StrList: TStringList
                    ): Boolean;

var
  P: PChar;
begin
  //結果
  result := FALSE;
  //判定
  if (Length(SearchString) = 0) or (SysUtils.StrLen(TextLine) = 0) then
  begin
    Exit;
  end;

  //検索
  P := SearchBuf5(TextLine, Info, SearchString, Options, StrList);
  //該当あり
  if P <> nil then
  begin
    result := TRUE;
  end;
end;

//TStrings に対して検索を行う関数。
//Info.Line ..... 検索を開始する行番号（０ベース）を指定する。
//Info.Column ... 検索を開始する桁番号（０ベース）を指定する。
//Info.Length ... 発見したときの文字列長さが格納される。
//選択状態の場合は、選択領域終端を渡すこと。
//上方向検索には対応していない。
//sfrIncludeCRLF も無視される。
function SearchStrings(Strings: TStrings;
                       var StringInfo: TStringsSearchInfo;
                       const SearchString: String;
                       Options: TSearchOptions;
                       MemoEx: TMemoExDark): Boolean;
var
  SearchInfo: TSearchInfo;
  I: Integer;
begin
  //結果
  result := FALSE;
  //設定
  Options := Options + [sfrDown] - [sfrIncludeCRLF];
  SearchInfo.Length := 0;

  //検索
  for I := StringInfo.Line to Strings.Count - 1 do
  begin
    //同一行
    if I = StringInfo.Line then
    begin
      SearchInfo.Start := StringInfo.Column
    end
    //異なった行
    else
    begin
      SearchInfo.Start := 0;
    end;

    //検索
    if SearchText1(PChar(Strings[I]), SearchInfo, SearchString, Options, MemoEx) = TRUE  then
    begin
      //結果
      StringInfo.Line := I;
      StringInfo.Column := SearchInfo.Start;
      StringInfo.Length := SearchInfo.Length;
      result := TRUE;
      Break;
    end;
  end;
end;


end.
